#include <Arduino.h>

#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <EEPROM.h>

#include "defines.h"
#include "configuration.h"


tConfigBlock configBlock;
const uint32_t MAGIC = 0xC0DE0001;
extern ESP8266WebServer webServer;

bool configSaved = false;

void configServeIndex() {
  bool configValid = (configBlock.magic == MAGIC);

  if (! configValid) {
    strcpy(configBlock.wifiSsid, "test");
    strcpy(configBlock.wifiKey, "geheim");
    strcpy(configBlock.mqttBroker, "broker.hottis.de");
    strcpy(configBlock.mqttUser, "TouchSwitch");
    strcpy(configBlock.mqttPass, "geheim123");
    strcpy(configBlock.mqttClientId, "TouchSwitch");
    strcpy(configBlock.mqttTopic, "IoT/TouchSwitch/Press");
    configBlock.mqttPort = 8883;
    configBlock.debugMode = 0;
    strcpy(configBlock.mqttDebugTopic, "IoT/TouchSwitch/Debug");
    configBlock.period = 500;
    configBlock.threshold = 350;
  }
  
  String buffer =
  "<!doctype html"
  "<html lang=\"en\">"
  "  <head>"
  "    <title>ESP8266 Thermometer Configuration Page</title>"
  "  </head>"
  "  <body>"
  "    <h1>ESP8266 Configuration Page</h1>";
  
  if (configSaved) {
    configSaved = false;
    buffer += "<h2>Configuration saved</h2>";
  }  
  
  buffer +=
  "    <form action=\"/config\" method=\"GET\">"
  "      <table>"
  "        <tr>"
  "          <td>"
  "            <label for\"wifiSsid\">Wifi SSID</label>"
  "          </td><td>"
  "            <input type=\"text\" name=\"wifiSsid\" id=\"wifiSsid\" ";
  
  buffer += "    size=\"32\" ";
  buffer += "    value=\"";
  buffer += configBlock.wifiSsid; 
  buffer += "\"";
  
  buffer +=
  "            />"
  "          </td>"
  "        </tr>"
  "        <tr>"
  "          <td>"
  "            <label for\"wifiKey\">Wifi Key</label>"
  "          </td><td>"
  "            <input type=\"text\" name=\"wifiKey\" id=\"wifiKey\" ";
  
  buffer += "    size=\"64\" ";
  buffer += "    value=\"";
  buffer += configBlock.wifiKey; 
  buffer += "\"";
  
  buffer +=
  "            />"
  "          </td>"
  "        </tr>"
  "        <tr>"
  "          <td>"
  "            <label for\"mqttBroker\">MQTT Broker</label>"
  "          </td><td>"
  "            <input type=\"text\" name=\"mqttBroker\" id=\"mqttBroker\" ";
  
  buffer += "    size=\"64\" ";
  buffer += "    value=\"";
  buffer += configBlock.mqttBroker; 
  buffer += "\"";
  
  buffer +=
  "            />"
  "          </td>"
  "        </tr>"
  "        <tr>"
  "          <td>"
  "            <label for\"mqttUser\">MQTT Username</label>"
  "          </td><td>"
  "            <input type=\"text\" name=\"mqttUser\" id=\"mqttUser\" ";
  
  buffer += "    size=\"32\" ";
  buffer += "    value=\"";
  buffer += configBlock.mqttUser; 
  buffer += "\"";
  
  buffer +=
  "            />"
  "          </td>"
  "        </tr>"
  "        <tr>"
  "          <td>"
  "            <label for\"mqttPass\">MQTT Password</label>"
  "          </td><td>"
  "            <input type=\"text\" name=\"mqttPass\" id=\"mqttPass\" ";
  
  buffer += "    size=\"32\" ";
  buffer += "    value=\"";
  buffer += configBlock.mqttPass; 
  buffer += "\"";
  
  buffer +=
  "            />"
  "          </td>"
  "        </tr>"
  "        <tr>"
  "          <td>"
  "            <label for\"mqttClientId\">MQTT ClientId</label>"
  "          </td><td>"
  "            <input type=\"text\" name=\"mqttClientId\" id=\"mqttClientId\" ";
  
  buffer += "    size=\"32\" ";
  buffer += "    value=\"";
  buffer += configBlock.mqttClientId; 
  buffer += "\"";
  
  buffer +=
  "            />"
  "          </td>"
  "        </tr>"
  "        <tr>"
  "          <td>"
  "            <label for\"mqttTopic\">MQTT Topic</label>"
  "          </td><td>"
  "            <input type=\"text\" name=\"mqttTopic\" id=\"mqttTopic\" ";
  
  buffer += "    size=\"64\" ";
  buffer += "    value=\"";
  buffer += configBlock.mqttTopic; 
  buffer += "\"";
  
  buffer +=
  "            />"
  "          </td>"
  "        </tr>"
  "        <tr>"
  "          <td>"
  "            <label for\"mqttPort\">MQTT Port</label>"
  "          </td><td>"
  "            <input type=\"text\" name=\"mqttPort\" id=\"mqttPort\" ";
  
  buffer += "    value=\""; 
  buffer += configBlock.mqttPort; 
  buffer += "\"";
  
  buffer +=
  "            />"
  "          </td>"
  "        </tr>"
  "        <tr>"
  "          <td>"
  "            <label for\"debugMode\">DebugMode</label>"
  "          </td><td>"
  "            <input type=\"text\" name=\"debugMode\" id=\"debugMode\" ";
  
  buffer += "    value=\""; 
  buffer += configBlock.debugMode; 
  buffer += "\"";
  
  buffer +=
  "            />"
  "          </td>"
  "        </tr>"
  "        <tr>"
  "          <td>"
  "            <label for\"mqttDebugTopic\">MQTT DebugTopic</label>"
  "          </td><td>"
  "            <input type=\"text\" name=\"mqttDebugTopic\" id=\"mqttDebugTopic\" ";
  
  buffer += "    size=\"64\" ";
  buffer += "    value=\"";
  buffer += configBlock.mqttDebugTopic; 
  buffer += "\"";
  
  buffer +=
  "            />"
  "          </td>"
  "        </tr>"
  "        <tr>"
  "          <td>"
  "            <label for\"period\">Period</label>"
  "          </td><td>"
  "            <input type=\"text\" name=\"period\" id=\"period\" ";
  
  buffer += "    value=\""; 
  buffer += configBlock.period; 
  buffer += "\"";
  
  buffer +=
  "            />"
  "          </td>"
  "        </tr>"
  "        <tr>"
  "          <td>"
  "            <label for\"threshold\">Threshold</label>"
  "          </td><td>"
  "            <input type=\"text\" name=\"threshold\" id=\"threshold\" ";
  
  buffer += "    value=\""; 
  buffer += configBlock.threshold; 
  buffer += "\"";
  
  buffer +=
  "            />"
  "          </td>"
  "        </tr>"
  "        <tr>"
  "          <td colspan=\"2\">"
  "            <button type=\"submit\">Save</button>"
  "          </td>"
  "        </tr>"
  "      </table>"
  "    </form>"
  "  </body>"
  "</html>";

  webServer.send(200, "text/html", buffer);


#ifdef DEBUG
  Serial.println("indexHtml request served");
#endif    
}

void configServeGetConfiguration() {
  String arg;

  arg = webServer.arg("wifiSsid");
  strcpy(configBlock.wifiSsid, arg.c_str());
  arg = webServer.arg("wifiKey");
  strcpy(configBlock.wifiKey, arg.c_str());
  arg = webServer.arg("mqttBroker");
  strcpy(configBlock.mqttBroker, arg.c_str());
  arg = webServer.arg("mqttUser");
  strcpy(configBlock.mqttUser, arg.c_str());
  arg = webServer.arg("mqttPass");
  strcpy(configBlock.mqttPass, arg.c_str());
  arg = webServer.arg("mqttClientId");
  strcpy(configBlock.mqttClientId, arg.c_str());
  arg = webServer.arg("mqttTopic");
  strcpy(configBlock.mqttTopic, arg.c_str());
  arg = webServer.arg("mqttPort");
  configBlock.mqttPort = atoi(arg.c_str());
  arg = webServer.arg("debugMode");
  configBlock.debugMode = atoi(arg.c_str());
  arg = webServer.arg("mqttDebugTopic");
  strcpy(configBlock.mqttDebugTopic, arg.c_str());
  arg = webServer.arg("period");
  configBlock.period = atoi(arg.c_str());
  arg = webServer.arg("threshold");
  configBlock.threshold = atoi(arg.c_str());
  
  configBlock.magic = MAGIC;
  
  showConfiguration();
  
  EEPROM.begin(512);
  EEPROM.put(EEPROM_ADDR, configBlock);  
  EEPROM.commit();
  
  Serial.println("EEPROM saved");

  configSaved = true;
  webServer.sendHeader("Location", String("/"), true);
  webServer.send(302, "text/plain", "");
  //webServer.send(200, "text/html", "configuration saved");
}
  
void showConfiguration() {
  Serial.println("Configuration is");

  Serial.print("magic = ");
  Serial.println(configBlock.magic);
    
  Serial.print("wifiSsid = ");
  Serial.println(configBlock.wifiSsid);
    
  Serial.print("wifiKey = ");
  Serial.println(configBlock.wifiKey);
    
  Serial.print("mqttBroker = ");
  Serial.println(configBlock.mqttBroker);
    
  Serial.print("mqttUser = ");
  Serial.println(configBlock.mqttUser);
    
  Serial.print("mqttPass = ");
  Serial.println(configBlock.mqttPass);
    
  Serial.print("mqttClientId = ");
  Serial.println(configBlock.mqttClientId);
    
  Serial.print("mqttTopic = ");
  Serial.println(configBlock.mqttTopic);
    
  Serial.print("mqttPort = ");
  Serial.println(configBlock.mqttPort);
    
  Serial.print("debugMode = ");
  Serial.println(configBlock.debugMode);
    
  Serial.print("mqttDebugTopic = ");
  Serial.println(configBlock.mqttDebugTopic);
    
  Serial.print("period = ");
  Serial.println(configBlock.period);
    
  Serial.print("threshold = ");
  Serial.println(configBlock.threshold);
    
  
  Serial.println("---");
}